%% Code for the ET algorithm in Den Haan et al. (2015)
% Solving the Coeurdacier et al (2011) with the ET algorithm 
%--------------------------------------------------------------------------

%Cleaning
clear all; %close all; clc
global rtil wtil ytil

%% Parameters
%--------------------------------------------------------------------------
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              
% options program
in_sup            = 'discrete';             % support for innovation, either discrete, or continuous

% parameter values
par.beta    = 0.96;
par.gamma   = 4;
par.ybar    = 1.0;
par.rhoy    = 0.9;
par.rhor    = 0.9;
par.sigy    = 0.01;
par.sigr    = 0.00125;
par.rbar    = 1.041528786850000;

% Limit values for y and r
par.cutoff = 4; % cut-off in terms of standard deviations
par.y_low  = par.ybar-par.cutoff*par.sigy/sqrt(1-par.rhoy^2);
par.y_high = par.ybar+par.cutoff*par.sigy/sqrt(1-par.rhoy^2);
par.r_low  = par.rbar-par.cutoff*par.sigr/sqrt(1-par.rhor^2);
par.r_high = par.rbar+par.cutoff*par.sigr/sqrt(1-par.rhor^2);

phi_num     = 10;       %number of coefficients in the Pontus approximation
par.T_simul = 80000;    %number if periods for the simulation

% options optimization & accuracy iteration
par.options   = optimset('Display','off','MaxFunEvals',1E7,'MaxIter',...
             1E7,'TolFun',1E-14,'TolX',1E-14);
         
%% parameter values for algorithm

%Gaussian hermite settings
gh.size  = 2;  [gh.e,gh.p] = hernodes(gh.size); %gh.e are the gh nodes and gh.p are the gh weights.

%% Setting up equations with symbolic toolbox

X = gh.e;
P = gh.p;
P = pi^(-1/2)*P;

syms r rm wlag w wm wconstm y ym phi_r phi_w phi_y phi_ww phi_rr phi_yy phi_rw phi_ry phi_wy

yprime = @(X) par.ybar*(1-par.rhoy) + par.rhoy*y + X*sqrt(2)*par.sigy;
rprime = @(X) par.rbar*(1-par.rhor) + par.rhor*r + X*sqrt(2)*par.sigr;

lhs = (y + r*wlag - w)^(-par.gamma); % left-hand side of the Euler equation

rhs = 0;

% a loop that calculates the right-hand side of the Euler equation,
% replacing the expectations term with numerical integration
for i = 1:gh.size
    for j = 1:gh.size
        rhs = rhs+P(i)*P(j)*par.beta*(    yprime(X(j)) + rprime(X(i))*w                              ...
                                   -( wconstm +     phi_r *(rprime(X(i)) - rm)                       ...
                                              +     phi_w *(w            - wm)                       ...
                                              +     phi_y *(yprime(X(j)) - ym)                       ...
                                              + 0.5*phi_rr*(rprime(X(i)) - rm)^2                     ...
                                              + 0.5*phi_ww*(w            - wm)^2                     ...
                                              + 0.5*phi_yy*(yprime(X(j)) - ym)^2                     ...
                                              + phi_rw*(rprime(X(i)) - rm)*(w            - wm)       ...
                                              + phi_ry*(rprime(X(i)) - rm)*(yprime(X(j)) - ym)       ...
                                              + phi_wy*(w            - wm)*(yprime(X(j)) - ym)       ...
                                     )                                                               ...                           
                                 )^(-par.gamma)                                                      ...
                                 *rprime(X(i));
    end
end

% Differentiating the Euler equation
% as a subscript w stands for wlag and W stands for w

eq    = 1 - rhs/lhs;

eq_W  = diff(eq,w);
eq_r  = diff(eq,r);
eq_w  = diff(eq,wlag);
eq_y  = diff(eq,y);
eq_WW = diff(eq_W,w);
eq_Wr = diff(eq_W,r);
eq_Ww = diff(eq_W,wlag);
eq_Wy = diff(eq_W,y);
eq_rr = diff(eq_r,r);
eq_ww = diff(eq_w,wlag);
eq_yy = diff(eq_y,y);
eq_rw = diff(eq_r,wlag);
eq_ry = diff(eq_r,y);
eq_wy = diff(eq_w,y);

% Equations allowing us to solve for phi's

dWdr     = -phi_r - eq_r/eq_W;
dWdw     = (phi_w*eq_W + eq_w)/(r-phi_w);
dWdy     = (phi_y*eq_W + eq_y)/(1-phi_y);
dWdrr    = eq_W*phi_rr + 2*eq_Wr*phi_r + eq_WW*phi_r^2 + eq_rr;
dWdww    = eq_W*phi_ww + 2*eq_Ww*phi_w + eq_WW*phi_w^2 + eq_ww;
dWdyy    = eq_W*phi_yy + 2*eq_Wy*phi_y + eq_WW*phi_y^2 + eq_yy;
dWdrw    = eq_W*phi_rw + eq_Ww*phi_r + eq_Wr*phi_w + eq_WW*phi_r*phi_w + eq_rw;
dWdry    = eq_W*phi_ry + eq_Wy*phi_r + eq_Wr*phi_y + eq_WW*phi_r*phi_y + eq_ry;
dWdwy    = eq_W*phi_wy + eq_Wy*phi_w + eq_Ww*phi_y + eq_WW*phi_w*phi_y + eq_wy;

% Saving the above equations as Matlab functions

f0   = matlabFunction(eq     ,'vars',{w, wconstm, rm, wm, ym, r, wlag, y, phi_r, phi_w, phi_y, phi_rr, phi_ww, phi_yy, phi_rw, phi_ry, phi_wy},'file','f0' );
fr   = matlabFunction(dWdr   ,'vars',{w, wconstm, rm, wm, ym, r, wlag, y, phi_r, phi_w, phi_y, phi_rr, phi_ww, phi_yy, phi_rw, phi_ry, phi_wy},'file','fr' );
fw   = matlabFunction(dWdw   ,'vars',{w, wconstm, rm, wm, ym, r, wlag, y, phi_r, phi_w, phi_y, phi_rr, phi_ww, phi_yy, phi_rw, phi_ry, phi_wy},'file','fw' );
fy   = matlabFunction(dWdy   ,'vars',{w, wconstm, rm, wm, ym, r, wlag, y, phi_r, phi_w, phi_y, phi_rr, phi_ww, phi_yy, phi_rw, phi_ry, phi_wy},'file','fy' );
frr  = matlabFunction(dWdrr  ,'vars',{w, wconstm, rm, wm, ym, r, wlag, y, phi_r, phi_w, phi_y, phi_rr, phi_ww, phi_yy, phi_rw, phi_ry, phi_wy},'file','frr');
fww  = matlabFunction(dWdww  ,'vars',{w, wconstm, rm, wm, ym, r, wlag, y, phi_r, phi_w, phi_y, phi_rr, phi_ww, phi_yy, phi_rw, phi_ry, phi_wy},'file','fww');
fyy  = matlabFunction(dWdyy  ,'vars',{w, wconstm, rm, wm, ym, r, wlag, y, phi_r, phi_w, phi_y, phi_rr, phi_ww, phi_yy, phi_rw, phi_ry, phi_wy},'file','fyy');
frw  = matlabFunction(dWdrw  ,'vars',{w, wconstm, rm, wm, ym, r, wlag, y, phi_r, phi_w, phi_y, phi_rr, phi_ww, phi_yy, phi_rw, phi_ry, phi_wy},'file','frw');
fry  = matlabFunction(dWdry  ,'vars',{w, wconstm, rm, wm, ym, r, wlag, y, phi_r, phi_w, phi_y, phi_rr, phi_ww, phi_yy, phi_rw, phi_ry, phi_wy},'file','fry');
fwy  = matlabFunction(dWdwy  ,'vars',{w, wconstm, rm, wm, ym, r, wlag, y, phi_r, phi_w, phi_y, phi_rr, phi_ww, phi_yy, phi_rw, phi_ry, phi_wy},'file','fwy');

disp('done with setting up the symbolic toolbox')

%% Simulations
%--------------------------------------------------------------------------

w_start = -5;
c_start = par.ybar + (par.rbar-1)*w_start;

TEMP1  = ones(phi_num,1);
TEMP2  = ones(phi_num,1);

rng(20140823,'twister')

e_y  = randn(par.T_simul,1);
e_r  = randn(par.T_simul,1);
e_yy = (e_y>0) - (e_y<0);
e_rr = (e_r>0) - (e_r<0);
e_y  = e_y *par.sigy;
e_yy = e_yy*par.sigy;
e_r  = e_r *par.sigr;
e_rr = e_rr*par.sigr;

% start setting things up for a regular simulation

simul.y         = zeros(par.T_simul,1);
simul.yy        = zeros(par.T_simul,1);
simul.r         = zeros(par.T_simul,1);
simul.rr        = zeros(par.T_simul,1);
simul.w         = zeros(par.T_simul,1);
simul.c         = zeros(par.T_simul,1);
simul.phi       = zeros(par.T_simul,phi_num);

simul.check     = zeros(par.T_simul,1);

simul.y(1)      = par.ybar;
simul.yy(1)     = par.ybar;
simul.r(1)      = par.rbar;
simul.rr(1)     = par.rbar;
simul.w(1,1)    = w_start;
simul.w(2,1)    = w_start;
simul.c(1,1)    = c_start;
simul.c(2,1)    = c_start;

simul.err       = zeros(par.T_simul,phi_num);

for t = 2:par.T_simul
    y_temp      = (1-par.rhoy)*par.ybar+par.rhoy*simul.y(t-1)  + e_y(t);
    yy_temp     = (1-par.rhoy)*par.ybar+par.rhoy*simul.yy(t-1) + e_yy(t);
    r_temp      = (1-par.rhor)*par.rbar+par.rhor*simul.r(t-1)  + e_r(t);
    rr_temp     = (1-par.rhor)*par.rbar+par.rhor*simul.rr(t-1) + e_rr(t);
    simul.y(t)  = min(  max( y_temp,par.y_low)   ,par.y_high);
    simul.yy(t) = min(  max(yy_temp,par.y_low)   ,par.y_high);
    simul.r(t)  = min(  max( r_temp,par.r_low)   ,par.r_high);
    simul.rr(t) = min(  max(rr_temp,par.r_low)   ,par.r_high);
end

% simul.y is the one that is actually used below
switch in_sup
    case 'discrete'
        simul.y = simul.yy;
        simul.r = simul.rr;
        disp('discrete support for innovation')
    case 'continuous'
        disp('continuous support for innovation')
end

A   = [];
b   = [];
Aeq = [];
beq = [];

start_0 = [ -4.9655    0.9928    0.7097   -2.3123   -1.6822    0.0000 0.0010    0.7836   -0.3326    0.0002]';

for t = 3:par.T_simul
    disp([100*t/par.T_simul, simul.w(t-1,1), simul.r(t), simul.y(t)])
    
    rtil = simul.r(t);
    ytil = simul.y(t);
    wtil = simul.w(t-1,1);

    lb = [simul.w(t-1,1)-0.5                        ,0.8 ,0.4, -30,-30,-10,-10,-10,-10, -10 ];
    ub = [simul.y(t)+simul.r(t)*simul.w(t-1,1)-0.05 ,1.03,0.9,  30, 30,+10,+10,+10,+10, +10 ];

    [TEMP2, info1, rc] = knitromatlab(@func_obj,start_0, A, b, Aeq, beq, lb, ub,@func_nlcon2,[],par.options);

    start_0         = TEMP2(:,1);
    simul.phi(t,:)  = start_0'; 
    simul.w(t,1)    = TEMP2(1,1);
    simul.c(t,1)    = simul.y(t) + simul.w(t-1,1)*simul.r(t) - simul.w(t,1);
    simul.check(t)  = rc;
     
    disp([t/par.T_simul t/1000 simul.w(t,1),simul.w(t,1)])
    disp(simul.phi(t-1,:))
    disp(simul.phi(t,:))
    disp(rc)
end
